@@ -16,3 +16,5 @@
 #
 
 export QML_IMPORT_PATH=$PWD/modules
+export QML2_IMPORT_PATH=$PWD/modules
+export UBUNTU_UI_TOOLKIT_THEMES_PATH=$PWD/modules
