@@ -19,8 +19,11 @@
 #include "shapeitem.h"
 #include "shapeitemtexture.h"
 #include "ucunits.h"
+#include <QtCore/QPointer>
 #include <QtQuick/QQuickWindow>
 #include <QtQuick/QSGTextureProvider>
+#include <QtQuick/private/qquickimage_p.h>
+
 
 // Retrieves the size of an array at compile time.
 #define ARRAY_SIZE(a) \
@@ -60,24 +63,15 @@ static const char* const shapeTexturedFragmentShader =
 static const char* const shapeColoredFragmentShader =
     "uniform lowp float opacity;                                                                \n"
     "uniform sampler2D shapeTexture;                                                            \n"
-    "uniform lowp vec4 baseColor;                                                               \n"
+    "uniform lowp vec4 color;                                                               \n"
     "uniform lowp vec4 gradientColor;                                                           \n"
     "varying lowp vec2 shapeCoord;                                                              \n"
     "varying lowp vec2 imageCoord;                                                              \n"
-    "lowp vec3 blendOverlay(lowp vec3 base, lowp vec3 blend)                                    \n"
-    "{                                                                                          \n"
-    "    lowp vec3 comparison = clamp(sign(base.rgb - vec3(0.5)), vec3(0.0), vec3(1.0));        \n"
-    "    return mix(2.0 * base * blend, 1.0 - 2.0 * (1.0 - base) * (1.0 - blend), comparison);  \n"
-    "}                                                                                          \n"
     "void main(void)                                                                            \n"
     "{                                                                                          \n"
     "    lowp vec4 shapeData = texture2D(shapeTexture, shapeCoord);                             \n"
-    "    lowp vec4 gradient = gradientColor * imageCoord.t;                                     \n"
-    "    lowp vec4 result = vec4(blendOverlay(baseColor.rgb / max(1.0/256.0, baseColor.a),      \n"
-    "                                         gradient.rgb / max(1.0/256.0, gradient.a)), 1.0); \n"
-    "    result *= baseColor.a;                                                                 \n"
-    "    lowp vec4 color = mix(baseColor, result, gradient.a) * vec4(shapeData.b);              \n"
-    "    lowp vec4 blend = shapeData.gggr + vec4(1.0 - shapeData.r) * color;                    \n"
+    "    lowp vec4 result = mix(color, gradientColor, imageCoord.t) * vec4(shapeData.b);        \n"
+    "    lowp vec4 blend = shapeData.gggr + vec4(1.0 - shapeData.r) * result;                   \n"
     "    gl_FragColor = blend * vec4(opacity);                                                  \n"
     "}";
 
@@ -135,12 +129,12 @@ static int sizeOfType(GLenum type)
 
 ShapeItem::ShapeItem(QQuickItem* parent)
     : QQuickItem(parent)
-    , baseColor_(0.0, 0.0, 0.0, 0.0)
+    , color_(0.0, 0.0, 0.0, 0.0)
     , gradientColor_(0.0, 0.0, 0.0, 0.0)
+    , gradientColorSet_(false)
     , radiusString_("small")
     , radius_(ShapeItem::SmallRadius)
-    , borderSource_("")
-    , border_(ShapeItem::RawBorder)
+    , border_(ShapeItem::IdleBorder)
     , image_(NULL)
     , stretched_(true)
     , hAlignment_(ShapeItem::AlignHCenter)
@@ -149,24 +143,33 @@ ShapeItem::ShapeItem(QQuickItem* parent)
     , geometry_()
     , dirtyFlags_(ShapeItem::DirtyAll)
 {
+    setFlag(ItemHasContents);
     QObject::connect(&UCUnits::instance(), SIGNAL(gridUnitChanged()), this,
                      SLOT(gridUnitChanged()));
-    setFlag(ItemHasContents);
+    setImplicitWidth(8 * gridUnit_);
+    setImplicitHeight(8 * gridUnit_);
     update();
 }
 
-void ShapeItem::setBaseColor(const QColor& baseColor)
+void ShapeItem::setColor(const QColor& color)
 {
-    if (baseColor_ != baseColor) {
-        baseColor_ = baseColor;
-        dirtyFlags_ |= ShapeItem::DirtyBaseColor;
+    if (color_ != color) {
+        color_ = color;
+        dirtyFlags_ |= ShapeItem::DirtyColor;
+        // gradientColor has the same value as color unless it was manually set
+        if (!gradientColorSet_) {
+            gradientColor_ = color;
+            dirtyFlags_ |= ShapeItem::DirtyGradientColor;
+            Q_EMIT gradientColorChanged();
+        }
         update();
-        Q_EMIT baseColorChanged();
+        Q_EMIT colorChanged();
     }
 }
 
 void ShapeItem::setGradientColor(const QColor& gradientColor)
 {
+    gradientColorSet_ = true;
     if (gradientColor_ != gradientColor) {
         gradientColor_ = gradientColor;
         dirtyFlags_ |= ShapeItem::DirtyGradientColor;
@@ -202,11 +205,19 @@ void ShapeItem::setBorderSource(const QString& borderSource)
     }
 }
 
-void ShapeItem::setImage(QVariant image)
+void ShapeItem::setImage(const QVariant& image)
 {
     QQuickItem* newImage = qobject_cast<QQuickItem*>(qvariant_cast<QObject*>(image));
     if (image_ != newImage) {
         image_ = newImage;
+
+        // update values of properties that depend on properties of the image
+        QObject::disconnect(image_);
+        if (newImage != NULL) {
+            updateFromImageProperties(newImage);
+            connectToImageProperties(newImage);
+        }
+
         if (image_ && !image_->parentItem()) {
             // Inlined images need a parent and must not be visible.
             image_->setParentItem(this);
@@ -218,6 +229,68 @@ void ShapeItem::setImage(QVariant image)
     }
 }
 
+void ShapeItem::updateFromImageProperties(QQuickItem* image)
+{
+    int alignment;
+
+    // ShapeItem::stretched depends on image::fillMode
+    QQuickImage::FillMode fillMode = (QQuickImage::FillMode)image->property("fillMode").toInt();
+    if (fillMode == QQuickImage::PreserveAspectCrop) {
+        setStretched(false);
+    } else {
+        setStretched(true);
+    }
+
+    // ShapeItem::horizontalAlignment depends on image::horizontalAlignment
+    int imageHorizontalAlignment = image->property("horizontalAlignment").toInt();
+    if (imageHorizontalAlignment == Qt::AlignLeft) {
+        alignment = ShapeItem::AlignLeft;
+    } else if (imageHorizontalAlignment == Qt::AlignRight) {
+        alignment = ShapeItem::AlignRight;
+    } else {
+        alignment = ShapeItem::AlignHCenter;
+    }
+    setHorizontalAlignment(static_cast<ShapeItem::HAlignment>(alignment));
+
+    // ShapeItem::verticalAlignment depends on image::verticalAlignment
+    int imageVerticalAlignment = image->property("verticalAlignment").toInt();
+    if (imageVerticalAlignment == Qt::AlignTop) {
+        alignment = ShapeItem::AlignTop;
+    } else if (imageVerticalAlignment == Qt::AlignBottom) {
+        alignment = ShapeItem::AlignBottom;
+    } else {
+        alignment = ShapeItem::AlignVCenter;
+    }
+    setVerticalAlignment(static_cast<ShapeItem::VAlignment>(alignment));
+}
+
+void ShapeItem::connectToPropertyChange(QObject* sender, const char* property,
+                                        QObject* receiver, const char* slot)
+{
+    int propertyIndex = sender->metaObject()->indexOfProperty(property);
+    if (propertyIndex != -1) {
+        QMetaMethod changeSignal = sender->metaObject()->property(propertyIndex).notifySignal();
+
+        int slotIndex = receiver->metaObject()->indexOfSlot(slot);
+        QMetaMethod updateSlot = receiver->metaObject()->method(slotIndex);
+
+        QObject::connect(sender, changeSignal, receiver, updateSlot);
+    }
+}
+
+void ShapeItem::connectToImageProperties(QQuickItem* image)
+{
+    connectToPropertyChange(image, "fillMode", this, "onImagePropertiesChanged()");
+    connectToPropertyChange(image, "horizontalAlignment", this, "onImagePropertiesChanged()");
+    connectToPropertyChange(image, "verticalAlignment", this, "onImagePropertiesChanged()");
+}
+
+void ShapeItem::onImagePropertiesChanged()
+{
+    QQuickItem* image = qobject_cast<QQuickItem*>(sender());
+    updateFromImageProperties(image);
+}
+
 void ShapeItem::setStretched(bool stretched)
 {
     if (stretched_ != stretched) {
@@ -251,6 +324,8 @@ void ShapeItem::setVerticalAlignment(VAlignment vAlignment)
 void ShapeItem::gridUnitChanged()
 {
     gridUnit_ = UCUnits::instance().gridUnit();
+    setImplicitWidth(8 * gridUnit_);
+    setImplicitHeight(8 * gridUnit_);
     dirtyFlags_ |= ShapeItem::DirtyGridUnit;
     update();
 }
@@ -269,15 +344,35 @@ QSGNode* ShapeItem::updatePaintNode(QSGNode* old_node, UpdatePaintNodeData* data
 
     // FIXME(loicm) Shape textures are stored in the read-only data section of the plugin as it
     //     avoids having to deal with paths for now. It should preferably be loaded from a file.
-    static bool once = false;
-    if (!once) {
+
+    /* Textures created with QWindow::createTextureFromImage() become invalid
+     * when the window is destroyed; therefore, we must keep track of which
+     * window was used to create them and be ready to re-create them if that
+     * window goes away. */
+    static QPointer<QWindow> textureOwner = 0;
+    if (!textureOwner) {
         shapeTextureHigh.texture = window()->createTextureFromImage(
             QImage(shapeTextureHigh.data, shapeTextureHigh.width, shapeTextureHigh.height,
                    QImage::Format_ARGB32_Premultiplied));
         shapeTextureLow.texture = window()->createTextureFromImage(
             QImage(shapeTextureLow.data, shapeTextureLow.width, shapeTextureLow.height,
                    QImage::Format_ARGB32_Premultiplied));
-        once = true;
+        textureOwner = window();
+    }
+
+    // The image item sets its texture in its updatePaintNode() method when QtQuick iterates through
+    // the list of dirty items. When we're notified the image item has been changed through
+    // setImage(), we mark the shape item as dirty by requesting an update. But sometimes it leads
+    // to have the shape item being queued in the dirty list before the image item. That case can be
+    // detected when the texture provider exists but not the texture itself. When that's the case we
+    // push the shape item in the dirty list to be handled next frame and we tell QtQuick not to
+    // render the item for the current frame.
+    const QSGTextureProvider* provider = image_ ? image_->textureProvider() : NULL;
+    const QSGTexture* texture = provider ? provider->texture() : NULL;
+    if (provider && !texture) {
+        update();
+        delete old_node;
+        return NULL;
     }
 
     ShapeNode* node = static_cast<ShapeNode*>(old_node);
@@ -310,7 +405,7 @@ QSGNode* ShapeItem::updatePaintNode(QSGNode* old_node, UpdatePaintNodeData* data
     texturedMaterial->setShapeTexture(textureData->texture, !!scaledDown);
 
     // Update the other material properties.
-    coloredMaterial->setBaseColor(baseColor_);
+    coloredMaterial->setColor(color_);
     coloredMaterial->setGradientColor(gradientColor_);
     texturedMaterial->setImage(image_);
 
@@ -339,6 +434,8 @@ ShapeNode::ShapeNode(ShapeItem* item)
     memcpy(geometry_.indexData(), shapeMesh.indices,
            shapeMesh.indexCount * sizeOfType(shapeMesh.indexType));
     geometry_.setDrawingMode(GL_TRIANGLE_STRIP);
+    geometry_.setIndexDataPattern(QSGGeometry::StaticPattern);
+    geometry_.setVertexDataPattern(QSGGeometry::AlwaysUploadPattern);
     setGeometry(&geometry_);
     setMaterial(&coloredMaterial_);
     setFlag(UsePreprocess, false);
@@ -360,6 +457,9 @@ void ShapeNode::setVertices(const QRectF& geometry, float radius, QQuickItem* im
     float radiusCoordinateWidth;
     float radiusCoordinateHeight;
 
+    // FIXME(loicm) With a NxM image, a preserve aspect crop fill mode and a width
+    //     component size of N (or a height component size of M), changing the the
+    //     height (or width) breaks the 1:1 texel/pixel mapping for odd values.
     if (!stretched && texture) {
         // Preserve source image aspect ratio cropping areas exceeding destination rectangle.
         const float factors[3] = { 0.0f, 0.5f, 1.0f };
@@ -373,8 +473,8 @@ void ShapeNode::setVertices(const QRectF& geometry, float radius, QQuickItem* im
             bottomCoordinate = 1.0f;
             leftCoordinate = outCoordinateSize * factors[hAlignment];
             rightCoordinate = 1.0f - (outCoordinateSize * (1.0f - factors[hAlignment]));
-            radiusCoordinateHeight = (radius - 1.0f) / (height - 1.0f);
-            radiusCoordinateWidth = ((radius - 1.0f) / (width - 1.0f)) * inCoordinateSize;
+            radiusCoordinateHeight = radius / height;
+            radiusCoordinateWidth = (radius / width) * inCoordinateSize;
         } else {
             const float inCoordinateSize = srcRatio / dstRatio;
             const float outCoordinateSize = 1.0f - inCoordinateSize;
@@ -382,8 +482,8 @@ void ShapeNode::setVertices(const QRectF& geometry, float radius, QQuickItem* im
             bottomCoordinate = 1.0f - (outCoordinateSize * (1.0f - factors[vAlignment]));
             leftCoordinate = 0.0f;
             rightCoordinate = 1.0f;
-            radiusCoordinateHeight = ((radius - 1.0f) / (height - 1.0f)) * inCoordinateSize;
-            radiusCoordinateWidth = (radius - 1.0f) / (width - 1.0f);
+            radiusCoordinateHeight = (radius / height) * inCoordinateSize;
+            radiusCoordinateWidth = radius / width;
         }
     } else {
         // Don't preserve source image aspect ratio stretching it in destination rectangle.
@@ -391,8 +491,19 @@ void ShapeNode::setVertices(const QRectF& geometry, float radius, QQuickItem* im
         bottomCoordinate = 1.0f;
         leftCoordinate = 0.0f;
         rightCoordinate = 1.0f;
-        radiusCoordinateHeight = (radius - 1.0f) / (height - 1.0f);
-        radiusCoordinateWidth = (radius - 1.0f) / (width - 1.0f);
+        radiusCoordinateHeight = radius / height;
+        radiusCoordinateWidth = radius / width;
+    }
+
+    // Scale and translate coordinates of textures packed in an atlas.
+    if (texture && texture->isAtlasTexture()) {
+        const QRectF srcSubRect = texture->normalizedTextureSubRect();
+        topCoordinate = topCoordinate * srcSubRect.height() + srcSubRect.y();
+        bottomCoordinate = bottomCoordinate * srcSubRect.height() + srcSubRect.y();
+        leftCoordinate = leftCoordinate * srcSubRect.width() + srcSubRect.x();
+        rightCoordinate = rightCoordinate * srcSubRect.width() + srcSubRect.x();
+        radiusCoordinateHeight = radiusCoordinateHeight * srcSubRect.height();
+        radiusCoordinateWidth = radiusCoordinateWidth * srcSubRect.width();
     }
 
     // Set top row of 4 vertices.
@@ -406,7 +517,7 @@ void ShapeNode::setVertices(const QRectF& geometry, float radius, QQuickItem* im
     vertices[1].position[1] = 0.0f;
     vertices[1].shapeCoordinate[0] = shapeCoordinate[1][0];
     vertices[1].shapeCoordinate[1] = shapeCoordinate[1][1];
-    vertices[1].imageCoordinate[0] = radiusCoordinateWidth;
+    vertices[1].imageCoordinate[0] = leftCoordinate + radiusCoordinateWidth;
     vertices[1].imageCoordinate[1] = topCoordinate;
     vertices[2].position[0] = width - radius;
     vertices[2].position[1] = 0.0f;
@@ -509,6 +620,7 @@ void ShapeNode::setMaterialType(ShapeNode::MaterialType material)
             setMaterial(&coloredMaterial_);
         else
             setMaterial(&texturedMaterial_);
+        currentMaterial_ = material;
         markDirty(DirtyMaterial);
     }
 }
@@ -616,7 +728,7 @@ void ShapeTexturedShader::updateState(const RenderState& state, QSGMaterial* new
 // --- Scene graph colored material ---
 
 ShapeColoredMaterial::ShapeColoredMaterial()
-    : baseColor_(0.0, 0.0, 0.0, 0.0)
+    : color_(0.0, 0.0, 0.0, 0.0)
     , gradientColor_(0.0, 0.0, 0.0, 0.0)
     , shapeTexture_(NULL)
     , filtering_(QSGTexture::Nearest)
@@ -635,12 +747,12 @@ QSGMaterialShader* ShapeColoredMaterial::createShader() const
     return new ShapeColoredShader;
 }
 
-void ShapeColoredMaterial::setBaseColor(const QColor& baseColor)
+void ShapeColoredMaterial::setColor(const QColor& color)
 {
     // Premultiply color components by alpha.
-    const float alpha = baseColor.alphaF();
-    baseColor_ = QVector4D(baseColor.redF() * alpha, baseColor.greenF() * alpha,
-                           baseColor.blueF() * alpha, alpha);
+    const float alpha = color.alphaF();
+    color_ = QVector4D(color.redF() * alpha, color.greenF() * alpha,
+                           color.blueF() * alpha, alpha);
 }
 
 void ShapeColoredMaterial::setGradientColor(const QColor& gradientColor)
@@ -684,7 +796,7 @@ void ShapeColoredShader::initialize()
     program()->setUniformValue("shapeTexture", 0);
     matrixId_ = program()->uniformLocation("matrix");
     opacityId_ = program()->uniformLocation("opacity");
-    baseColorId_ = program()->uniformLocation("baseColor");
+    colorId_ = program()->uniformLocation("color");
     gradientColorId_ = program()->uniformLocation("gradientColor");
 }
 
@@ -710,6 +822,6 @@ void ShapeColoredShader::updateState(const RenderState& state, QSGMaterial* newE
         program()->setUniformValue(matrixId_, state.combinedMatrix());
     if (state.isOpacityDirty())
         program()->setUniformValue(opacityId_, state.opacity());
-    program()->setUniformValue(baseColorId_, material->baseColor());
+    program()->setUniformValue(colorId_, material->color());
     program()->setUniformValue(gradientColorId_, material->gradientColor());
 }
