@@ -1,6 +1,6 @@
 #!/bin/bash
 #
-# Copyright 2012 Canonical Ltd.
+# Copyright 2012, 2013 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,10 +19,11 @@ cd tests/autopilot
 
 echo running with arg: $1
 
+UBUNTU_UI_TOOLKIT_AUTOPILOT_FROM_SOURCE=1
 if [ "$1" == "" ]; then
-	autopilot run ubuntu-ui-toolkit
+	python3 -m autopilot.run run ubuntuuitoolkit
 else
-	autopilot run -o ../../$1 -f xml -r -rd ../../ ubuntu-ui-toolkit
+	python3 -m autopilot.run run -o ../../$1 -f xml -r -rd ../../ ubuntuuitoolkit
 fi
 
 exit 0
